@@ -7,21 +7,19 @@ import mimetypes
 import rdflib
 import socket
 import urlparse
-from rdflib.namespace import split_uri
-from rdflib import compare
 from xml.sax._exceptions import SAXParseException
 
-# socket timeout set to 5 seconds globally
-socket.setdefaulttimeout(5)
+from rdflib.namespace import split_uri
+from rdflib import compare
+
+import utils
+
+# set socket timeout. URLError will occur
+socket.setdefaulttimeout(20)
 
 logger = logging.getLogger(__name__)
 
 
-class UriNotValid(Exception):
-    "Given Uri is not valid"
-    silent_variable_failure = True
-
-
 def safe_dict(d):
     """Recursively clone json structure with UTF-8 dictionary keys"""
     if isinstance(d, dict):
@@ -42,56 +40,6 @@ def catchKeyboardInterrupt(func):
     return dec
 
 
-def canonalize_uri(uriref):
-    """Returns Uri that is valid and canonalized or raises Exception"""
-    if not uriref:
-        raise UriNotValid("uri is None")
-
-    # TODO: implement canonalization: cut port 80, lowercase domain,
-    # http and https is equal. Problem: graph comparison
-
-    # Workaround for rdflib's handling of BNodes
-    if isinstance(uriref, rdflib.BNode):
-        return uriref
-
-    assert not isinstance(uriref, rdflib.Literal)
-
-    if not uriref.encode('utf8'):
-        raise UriNotValid("Not valid: %s" % uriref)
-
-    if not hasattr(uriref, "n3"):
-        logger.debug(u"Converting %s to URIRef" % uriref)
-        uriref = rdflib.URIRef(uriref)
-
-    if uriref.startswith('#'):
-        raise UriNotValid("%s starts with '#'. Check your Parser" % uriref)
-
-    return uriref
-
-
-def hash_to_slash_uri(uri):
-    """Converts Hash to Slash uri http://www.w3.org/wiki/HashURI"""
-    if not isinstance(uri, rdflib.URIRef):
-        logger.error("Cannot analyse %s (type %s)" % (uri, type(uri)))
-        return uri
-
-    parsed = urlparse.urlparse(uri)
-    uri = urlparse.urlunparse((parsed.scheme, parsed.netloc, parsed.path,
-                         parsed.params, parsed.query,""))
-    return uri
-
-
-def convert_to_rdflib_object(v):
-    if hasattr(v, "n3"):
-        return v
-
-    if hasattr(v, "startswith") and v.startswith("http://"):
-        # float has no attribute startswith
-        return rdflib.URIRef(v)
-    else:
-        return rdflib.Literal(v)
-
-
 def reverse_dict(dct):
     res = {}
     for k,v in dct.iteritems():
@@ -164,17 +112,26 @@ class ResourceManager(Manager):
         return origin_uri+uri
 
     def create(self, uri, origin, **kwargs):
+
         assert isinstance(origin, Origin), "Origin instance required"
         assert origin.processed, ("Origin has to be processed "
             "before creating more Resource objects: origin.GET()")
 
-        # Absolutize relative URIs
-        if uri.startswith("#"):
-            assert not isinstance(uri, rdflib.BNode), \
-                "BNode not allowed to start with #"
-            uri = rdflib.URIRef(origin.uri + uri)
+        # TODO write testcase for this first
+        ## Absolutize relative URIs
+        #if uri.startswith("#"):
+        #    assert not isinstance(uri, rdflib.BNode), \
+        #        "BNode not allowed to start with #"
+        #    uri = rdflib.URIRef(origin.uri + uri)
 
-        uri = canonalize_uri(uri)
+        if isinstance(uri, (rdflib.BNode, rdflib.URIRef)):
+            pass
+        elif not utils.is_valid_url(uri):
+            msg = "URI %s not a valid URL" % uri
+            logger.error(msg)
+            raise ValueError(msg)
+
+        uri = utils.get_rdflib_uriref(uri)
 
         pk = self.get_pk(origin_uri=origin.uri, uri=uri)
         return super(ResourceManager, self).create(
@@ -183,8 +140,13 @@ class ResourceManager(Manager):
     def get_authoritative_resource(self, uri,
                                    create_nonexistent_origin=True):
         """Tries to return the Resource object from the original origin"""
-        uri = canonalize_uri(uri)
-        origin_uri = rdflib.URIRef(hash_to_slash_uri(uri))
+
+        if not utils.is_valid_url(uri):
+            logger.error("Not a valid Uri: %s" % uri)
+            return
+        uri = utils.get_rdflib_uriref(uri)
+
+        origin_uri = utils.hash_to_slash_uri(uri)
 
         authoritative_origin = Origin.objects.filter(uri=origin_uri)
         authoritative_origin_list = list(authoritative_origin)
@@ -199,7 +161,7 @@ class ResourceManager(Manager):
 
         # TODO: make this configurable
         if not origin.has_unsaved_changes():
-            origin.GET(only_follow_uris=[])
+            origin.GET(only_follow_uris=[], raise_errors=False)
 
         authoritative_resource = self.get(uri=uri, origin=origin)
         return authoritative_resource
@@ -215,7 +177,8 @@ class ResourceManager(Manager):
         --> If DoesNotExist occurs Resources with uri might still
         exist but no validated Resources exist.
         """
-        uri = canonalize_uri(uri)
+
+        uri = utils.get_rdflib_uriref(uri)
 
         if not origin:
             filter_result = list(self.filter(_uri=uri))
@@ -239,7 +202,8 @@ class ResourceManager(Manager):
         assert 0, "implement!"
 
     def get_or_create(self, uri, origin=None):
-        uri = canonalize_uri(uri)
+
+        uri = utils.get_rdflib_uriref(uri)
         try:
             return self.get(uri=uri, origin=origin), False
         except self.model.DoesNotExist:
@@ -269,11 +233,12 @@ class Resource(Model):
         if isinstance(self._uri, rdflib.BNode):
             return True
 
-        if hash_to_slash_uri(self._uri) == str(self._origin.uri):
+        if utils.hash_to_slash_uri(self._uri) == self._origin.uri:
             return True
 
     def _add_property(self, predicate, object,
                       namespace_short_notation_reverse_dict):
+
         assert isinstance(predicate, rdflib.URIRef),\
             "Not an URIRef: %s"%predicate
         assert hasattr(predicate, "n3"),\
@@ -333,6 +298,26 @@ class Resource(Model):
 
     def _tripleserialize_iterator(self,
                                   namespace_dict):
+
+        def do_yield(resource, property, v):
+            if isinstance(v, resource.__class__):
+                # If object is referenced in attribute, "de-reference"
+                return ((resource._uri, property, v._uri))
+            else:
+
+                if not hasattr(v, "n3"):
+                    # print only newly added values without the correct
+                    # type are handled here
+
+                    if (hasattr(v, "startswith") # float has no attribute startswith
+                        and v.startswith("http://") # TODO: this is not very accurate!
+                        ):
+                        v = rdflib.URIRef(v)
+                    else:
+                        v = rdflib.Literal(v)
+
+                return ((self._uri, property, v))
+
         for property, values in self.__dict__.items():
 
             # skip internals
@@ -344,27 +329,15 @@ class Resource(Model):
             else:
                 property = pyattr2predicate(property, namespace_dict)
 
-            assert hasattr(property, "n3"), \
-                "property %s is not a rdflib object" % property
+            assert isinstance(property, rdflib.URIRef), \
+                "property %s is not a URIRef object" % property
 
             if isinstance(values, set):
                 for v in values:
-                    if isinstance(v, self.__class__):
-                        # If object is referenced in attribute, "de-reference"
-                        yield((self._uri, property, v._uri))
-                    else:
-                        if not hasattr(v, "n3"):
-                            v = convert_to_rdflib_object(v)
-                        yield((self._uri, property, v))
+                    yield do_yield(self, property, v)
             else:
                 v = values
-                if isinstance(v, self.__class__):
-                    # If object is referenced in attribute, "de-reference"
-                    yield((self._uri, property, v._uri))
-                else:
-                    if not hasattr(v, "n3"):
-                        v = convert_to_rdflib_object(v)
-                    yield((self._uri, property, v))
+                yield do_yield(self, property, v)
 
     def __setattr__(self, key, value):
         if key == "_has_changes":
@@ -403,17 +376,16 @@ class Resource(Model):
 
 class OriginManager(Manager):
 
-    def create_hook(self, uri):
-        assert not '#' in uri, ("HashURI not allowed as Origin: %s" % uri)
-
     def post_create_hook(self, origin):
         # hook to be overwritten but using application
         # origin.timedelta = datetime.timedelta(minutes=2)
         return origin
 
     def create(self, uri, BACKEND=None):
-        uri = canonalize_uri(uri)
-        self.create_hook(uri)
+        uri = utils.get_rdflib_uriref(uri)
+
+        assert not '#' in uri, ("HashURI not allowed as Origin: %s" % uri)
+
         backend = BACKEND if BACKEND else RestBackend()
         origin = super(OriginManager, self).create(pk=uri, uri=uri,
                                                  backend=backend)
@@ -421,15 +393,26 @@ class OriginManager(Manager):
 
     def get(self, uri, **kwargs):
         """Retrieves Origin object from Store"""
-        uri = canonalize_uri(uri)
+
+        uri = utils.get_rdflib_uriref(uri)
+
         return super(OriginManager, self).get(pk=uri)
 
-    def get_or_create(self, uri, **kwargs):
-        uri = canonalize_uri(uri)
-        assert str(uri) == str(hash_to_slash_uri(uri))
+    def get_or_create(self, uri, fail_silently=True,
+                      **kwargs #TODO: remove kwargs for better api
+                      ):
+
+        uri = utils.get_rdflib_uriref(uri)
+        if not uri == utils.hash_to_slash_uri(uri):
+            msg = "URI is not a slash URI: %s" % uri
+            if fail_silently:
+                logger.error(msg)
+                return None, None
+            else: raise Exception(msg)
+
         try:
             if kwargs:
-                logger.warning("kwargs not supportet in get")
+                logger.warning("kwargs are ignored for get.")
             return self.get(uri), False
         except self.model.DoesNotExist:
             return self.create(uri, **kwargs), True
@@ -437,8 +420,9 @@ class OriginManager(Manager):
     @catchKeyboardInterrupt
     def GET_all(self, depth=2, **kwargs):
         """Crawls or Re-Crawls all Origins. Passes Arguments to GET"""
+
         # TODO: limit crawling speed
-        assert depth<5
+
         func = lambda origin: True if not origin.processed else False
         for _i in range(depth):
             crawl = filter(func, self.all())
@@ -523,6 +507,11 @@ class Origin(Model):
             self.add_error("timeout")
             if raise_errors: raise e
             else: return
+        except ContentNegotiationError as e:
+            logger.error(e.message)
+            if raise_errors: raise e
+            else: return
+
 
         graph = rdflib.graph.ConjunctiveGraph(identifier=self.uri)
 
@@ -538,7 +527,8 @@ class Origin(Model):
 
                 # normal rdflib.compare does not work correctly with
                 # ConjunctiveGraph, unless there is only one graph within that
-                assert len(list(graph.contexts())) == 1
+                if not self.uri in [rdflib.URIRef("http://dbpedia.org/resource/Spinner_(Blade_Runner)")]:
+                    assert len(list(graph.contexts())) == 1
 
         except SAXParseException as e:
             self.add_error("SAXParseException")
@@ -582,7 +572,6 @@ class Origin(Model):
             else:
                 logging.warning("GET retrieved updates for %s!" % self.uri)
                 try:
-                    import utils
                     utils.my_graph_diff(self._graph, graph)
                 except ImportError:
                     pass
@@ -658,7 +647,6 @@ class Origin(Model):
                     if (hasattr(resource, '_has_changes')
                         and resource._has_changes == True)):
             return True
-
         return False
 
     def PUT(self):
@@ -713,7 +701,8 @@ class GraphHandler(object):
 
         for subject, predicate, obj_ect in graph:
             assert hasattr(subject, "n3")
-            #subject = canonalize_uri(subject)
+
+            #subject = utils.get_rdflib_uriref(subject)
 
             #if isinstance(subject, rdflib.BNode):
             #    assert 0, "subject is BNode: %s %s %s" % (subject, predicate, obj_ect)
@@ -729,7 +718,8 @@ class GraphHandler(object):
                 if (predicate == rdflib.OWL.imports
                     and type(obj_ect) == rdflib.URIRef):
 
-                    uri = hash_to_slash_uri(obj_ect)
+                    uri = utils.hash_to_slash_uri(obj_ect)
+
                     origin, created = Origin.objects.get_or_create(uri=uri)
 
                     logger.info("Interrupting to process owl:imports %s"
@@ -739,8 +729,11 @@ class GraphHandler(object):
             if ((self.only_follow_uris is not None
                  and predicate in self.only_follow_uris)
                 or self.only_follow_uris is None):
+
                 if type(obj_ect) == rdflib.URIRef:
-                    Origin.objects.get_or_create(uri=hash_to_slash_uri(obj_ect))
+                    if utils.is_valid_url(obj_ect):
+                        obj_uriref = utils.hash_to_slash_uri(obj_ect, logger=logger)
+                        Origin.objects.get_or_create(uri=obj_uriref)
 
             resource, _created = Resource.objects.get_or_create(uri=subject,
                                                         origin=self.origin)
