@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-__version__ = "0.5.3"
+__version__ = "0.5.4"
 __useragent__ = ('ldtools-%s (http://github.com/dmr/ldtools, daniel@nwebs.de)'
                  % __version__)
 
@@ -12,21 +12,20 @@ import shutil
 import urllib2
 import glob
 
+
 # add n3 to known mimetypes
 mimetypes.add_type("text/n3", ".n3")
+mimetypes.add_type("text/rdf+n3", ".n3")
+
+
+class FiletypeMappingError(Exception): pass
+class ContentNegotiationError(Exception): pass
+
 
 logger = logging.getLogger("ldtools")
 
 
 def get_file_extension(filename):
-    """
-    >>> get_file_extension("test.xml")
-    'xml'
-    >>> get_file_extension("test.1234123.xml")
-    'xml'
-    >>> get_file_extension("test")
-    ''
-    """
     extension = filename.split(".")[1:][-1:]
     return str(extension[0]) if extension else ""
 
@@ -36,11 +35,11 @@ def assure_parser_plugin_exists(format):
     # TODO: move parser to backend
     try:
         rdflib.graph.plugin.get(name=format,
-                                kind=rdflib.parser.Parser)
-        return True
+                            kind=rdflib.parser.Parser)
     except rdflib.plugin.PluginException as e:
-        logger.error("Parser does not exist for format %s" % format)
-        return False
+        msg = "Not parser plugin found for %s" % format
+        logger.error(msg)
+        raise ContentNegotiationError(msg)
 
 
 def guess_format_from_filename(file_name):
@@ -56,8 +55,23 @@ class AbstractBackend(object):
 
 
 class RestBackend(AbstractBackend):
+    GET_headers = {
+        'User-agent': __useragent__,
+        'Accept': (
+            'text/n3,'
+            'text/rdf+n3,'
+            'application/rdf+xml;q=0.8'
+            #'application/xhtml+xml;q=0.5'
+            #'*/*;q=0.1'
+            #XHTML+RDFa
+        )
+    }
 
-    def GET(self, uri):
+    PUT_headers={
+        "User-Agent": __useragent__,
+    }
+
+    def GET(self, uri, extra_headers=None, debug=False):
         """lookup URI"""
         # TODO: do friendly crawling: use robots.txt speed
         # limitation definitions
@@ -69,68 +83,97 @@ class RestBackend(AbstractBackend):
                 raise Exception("You cannot pass different uris to the same "
                                 "backend")
 
-        headers = {'User-agent': __useragent__,
-                   'Accept':('text/n3;q=1,'
-                             'application/rdf+xml,text/rdf+n3;q=0.9,'
-                             'application/xhtml+xml;q=0.5, */*;q=0.1')}
+        # custom redirecthandler for testing purposes
+        #class CustomRedirectHandler(urllib2.HTTPRedirectHandler):
+        #    def redirect_request(self, req, fp, code, msg, headers, newurl):
+        #        print code; print msg; print headers; print newurl
+        #        return urllib2.HTTPRedirectHandler.redirect_request(self,
+        # req, fp, code, msg, headers, newurl)
+        #opener = urllib2.build_opener(CustomRedirectHandler())
+        #class SmartRedirectHandler(urllib2.HTTPRedirectHandler):
+        #    def http_error_302(self, req, fp, code, msg, headers):
+        #        result = urllib2.HTTPRedirectHandler.http_error_302(self, req,
+        #                                                fp, code, msg, headers)
+        #        result.status = code
+        #        result.headers = headers
+        #        return result
 
-        request = urllib2.Request(url=self.uri, headers=headers)
+        if debug:
+            handler=urllib2.HTTPHandler(debuglevel=1)
+            opener = urllib2.build_opener(handler)
+        else:
+            opener = urllib2.build_opener()
+
+        if extra_headers:
+            self.GET_headers.update(extra_headers)
+
+        request = urllib2.Request(url=uri, headers=self.GET_headers)
 
-        opener = urllib2.build_opener() #SmartRedirectHandler())
         result_file = opener.open(request)
 
+        if result_file.geturl() != uri:
+            logger.info("%s was redirected. Content url: %r" % (uri,
+                                                     result_file.geturl()))
+
+        if "Content-Length" in result_file.headers:
+            logger.info("Content-Length: %s"
+                        % result_file.headers["Content-Length"])
+
+        #print result_file.info()
+        #print "os.stat().st_size returns:", os.stat("out.txt").st_size
+        #f = result_file; f.seek(0,2); print f.tell()  #os.fstat(f.fileno())[6]
+
+        if not "Content-Type" in result_file.headers:
+            raise FiletypeMappingError("No Content-Type specified in response")
         self.content_type = result_file.headers['Content-Type'].split(";")[0]
 
         # Many servers don't do content negotiation: if one of the following
         # content_types are returned by server, assume the mapped type
-        content_type_mapping = {
+        overwrite_content_type_map = {
             "text/plain": "application/rdf+xml"
         }
+        if self.content_type in overwrite_content_type_map:
+            self.content_type = overwrite_content_type_map[self.content_type]
 
-        if self.content_type in content_type_mapping:
-            self.content_type = content_type_mapping[self.content_type]
+        try:
+            file_extension = mimetypes.guess_extension(self.content_type)
+            assert file_extension
+        except AssertionError as e:
+            logger.error("%s not supported by ldtools"
+                         % result_file.headers['Content-Type'])
+            raise FiletypeMappingError("No mimetype found for %s"
+                                   % self.content_type)
 
-        file_extension = mimetypes.guess_extension(self.content_type)
-        if file_extension:
-            format = file_extension.strip(".")
-            if format in ["rdf", "ksh"]:
-                # fix responses that we know are wrong
-                format = "xml"
+        format = file_extension.strip(".")
+        if format in ["rdf", "ksh"]:
+            # fix responses that we know are wrong
+            format = "xml"
 
-            # TODO: try rdfa for format = "html"?
+        assure_parser_plugin_exists(format)
 
-            if assure_parser_plugin_exists(format):
-                self.format = format
+        self.format = format
+        return result_file.read()
 
-                data = result_file.read()
-
-                return data
-        else:
-            #logger.warning("%s not supported by ldtools. Trying 'xml'..."
-            #               % result_file.headers['Content-Type'])
-            #self.format = "xml"
-            #self.content_type = mimetypes.types_map[".%s" % self.format]
-            # TODO: assumption here: empty string will not cause harm because
-            # Origin.GET() will break after this
-            return
 
     def PUT(self, data):
         assert self.uri, "GET has to be called before PUT possible"
 
+        self.PUT_headers.update({
+            "Content-Type": self.content_type,
+            "Content-Length": str(len(data)),
+        })
+
         # TODO: authentication? oauth?
         #h = httplib2.Http()
         #h.add_credentials('name', 'password')
         #resp, content = h.request(uri, "PUT", body=data, headers=headers)
         #if resp.status != 200: raise Error(resp.status, errmsg, headers)
         #return resp, content
-        headers={"content-type": self.content_type,
-                 "User-Agent": __useragent__,
-                 "Content-Length": str(len(data))}
 
         opener = urllib2.build_opener(urllib2.HTTPHandler)
         request = urllib2.Request(self.uri,
                                   data=data,
-                                  headers=headers)
+                                  headers=self.PUT_headers)
         request.get_method = lambda: 'PUT'
         response = opener.open(request)
         # assert status_code 200, 201?
@@ -145,11 +188,9 @@ class FileBackend(AbstractBackend):
         assert os.path.exists(filename)
 
         format = format if format else guess_format_from_filename(filename)
-        if assure_parser_plugin_exists(format):
-            self.format = format
-        else:
-            logger.error("No format set!")
+        assure_parser_plugin_exists(format)
 
+        self.format = format
         self.filename = filename
 
     def GET(self, uri):
@@ -195,6 +236,7 @@ class FileBackend(AbstractBackend):
 class MemoryBackend(AbstractBackend):
     def __init__(self, data=None, format="xml"):
         self.data = data if data else ""
+        assure_parser_plugin_exists(format)
         self.format = format
 
     def GET(self, uri):
